@@ -4,16 +4,14 @@
 
 use std::path::{Path, PathBuf};
 
-use log::{debug, trace};
-
 use ra_ap_cfg::{self as cfg};
-use ra_ap_hir::{self as hir, AsAssocItem as _, HasAttrs as _, HirFileIdExt as _};
+use ra_ap_hir::{self as hir, AsAssocItem as _, HasAttrs as _};
 use ra_ap_ide::{self as ide};
 use ra_ap_ide_db::{self as ide_db};
 use ra_ap_load_cargo::{self as load_cargo};
 use ra_ap_paths::{self as paths};
 use ra_ap_project_model::{self as project_model};
-use ra_ap_syntax::{self as syntax, ast, AstNode as _};
+use ra_ap_syntax::{self as syntax, AstNode as _, ast};
 use ra_ap_vfs::{self as vfs};
 
 use crate::{
@@ -46,7 +44,7 @@ pub fn load_workspace(
     let load_config = load_config();
 
     let progress = |string| {
-        trace!("Progress: {}", string);
+        tracing::trace!("Progress: {}", string);
     };
 
     let mut project_workspace = load_project_workspace(&project_path, &cargo_config, &progress)?;
@@ -90,16 +88,14 @@ pub fn cargo_config(
             global: cfg::CfgDiff::new(
                 vec![cfg::CfgAtom::Flag(hir::Symbol::intern("test"))],
                 Vec::new(),
-            )
-            .unwrap(),
+            ),
             selective: Default::default(),
         },
         false => project_model::CfgOverrides {
             global: cfg::CfgDiff::new(
                 Vec::new(),
                 vec![cfg::CfgAtom::Flag(hir::Symbol::intern("test"))],
-            )
-            .unwrap(),
+            ),
             selective: Default::default(),
         },
     };
@@ -123,6 +119,8 @@ pub fn cargo_config(
 
     let invocation_strategy = project_model::InvocationStrategy::PerWorkspace;
 
+    let no_deps: bool = false;
+
     let run_build_script_command = None;
 
     // Rustc private crate source
@@ -157,6 +155,7 @@ pub fn cargo_config(
         extra_includes,
         features,
         invocation_strategy,
+        no_deps,
         run_build_script_command,
         rustc_source,
         set_test,
@@ -209,11 +208,11 @@ pub fn select_package_and_target(
 
     let package_idx = select_package(cargo_workspace, options)?;
     let package = cargo_workspace[package_idx].clone();
-    debug!("Selected package: {:#?}", package.name);
+    tracing::debug!("Selected package: {:#?}", package.name);
 
     let target_idx = select_target(cargo_workspace, package_idx, options)?;
     let target = cargo_workspace[target_idx].clone();
-    debug!("Selected target: {:#?}", target.name);
+    tracing::debug!("Selected target: {:#?}", target.name);
 
     Ok((package, target))
 }
@@ -429,7 +428,7 @@ pub fn find_crate(
 
 pub(crate) fn crate_name(krate: hir::Crate, db: &ide::RootDatabase) -> String {
     // Obtain the crate's declaration name:
-    let display_name = &krate.display_name(db).unwrap();
+    let display_name = krate.display_name(db).unwrap().to_string();
 
     // Since a crate's name may contain `-` we canonicalize it by replacing with `_`:
     display_name.replace('-', "_")
@@ -547,11 +546,7 @@ pub(crate) fn path(
         path.push_str(&relative_path);
     }
 
-    if path.is_empty() {
-        None
-    } else {
-        Some(path)
-    }
+    if path.is_empty() { None } else { Some(path) }
 }
 
 fn assoc_item_path(
@@ -689,14 +684,14 @@ pub(crate) fn has_test_cfg(hir: hir::ModuleDef, db: &ide::RootDatabase) -> bool
                     CfgAtom::KeyValue { .. } => false,
                 }
             })
-            .is_some()
+            .unwrap_or_default()
     })
 }
 
 pub(crate) fn is_test_function(function: hir::Function, db: &ide::RootDatabase) -> bool {
     let attrs = function.attrs(db);
     let key = hir::Symbol::intern("test");
-    attrs.by_key(&key).exists()
+    attrs.by_key(key).exists()
 }
 
 pub fn cfgs(hir: hir::ModuleDef, db: &ide::RootDatabase) -> Vec<cfg::CfgExpr> {
@@ -763,7 +758,7 @@ pub fn module_file(module: hir::Module, db: &ide::RootDatabase, vfs: &vfs::Vfs)
     }
 
     let file_id = module_source.file_id.original_file(db);
-    let vfs_path = vfs.file_path(file_id.into());
+    let vfs_path = vfs.file_path(file_id.file_id(db));
     let abs_path = vfs_path.as_path().expect("Could not convert to path");
 
     let path: &Path = abs_path.as_ref();
